// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include <ft2build.h>
#include FT_FREETYPE_H
#include "util/flag.hpp"
#include "display/Texture.hpp"

#define TRUE_TYPE_CHAR_BEGIN 0x00
#define TRUE_TYPE_CHAR_END 0xFF

#define TRUE_TYPE_VARIANT_BOLD FLAG_DEFINE(0)
#define TRUE_TYPE_VARIANT_ITALICS FLAG_DEFINE(1)

#define TRUE_TYPE_DECORATION_STRIKETHROUGH FLAG_DEFINE(0)
#define TRUE_TYPE_DECORATION_UNDERLINE FLAG_DEFINE(1)

namespace Dawn {
  class TrueTypeAsset;

  struct TrueTypeCharacter {
    float_t advanceX;
    float_t advanceY;
    glm::vec2 bitmapSize;
    glm::vec2 bitmapPosition;
    float_t textureY;
  };

  struct TrueTypeFaceTextureStyle {
    uint32_t fontSize;
    flag_t style;

    bool operator < (const struct TrueTypeFaceTextureStyle& r) const {
      return std::tie(fontSize, style) < std::tie(r.fontSize, r.style);
    }
  };

  class TrueTypeFaceTexture {
    public:
      const FT_Face face;
      const struct TrueTypeFaceTextureStyle style;
      std::map<FT_ULong, struct TrueTypeCharacter> characterData;
      Texture texture;

      /**
       * Construct a new New True Type Face Texture object
       * 
       * @param face The freetype face object.
       * @param style Style that this font has, used for locking.
       */
      TrueTypeFaceTexture(
        const FT_Face face,
        const struct TrueTypeFaceTextureStyle style
      );

      /**
       * Returns the character data for the given character.
       * 
       * @param c Character to get data for.
       * @return The Character data for the given character.
       */
      struct TrueTypeCharacter getCharacterData(const FT_ULong c);

      /**
       * Destroys this true type face texture.
       */
      virtual ~TrueTypeFaceTexture();

      friend class TrueTypeAsset;
  };
}